--! @file
--! @brief Counter with overflow

-- Jan Pospisil
--   2010; KP, FEL, CVUT
--   2015; CTP, ALICE, CERN
--   2016; BE-BI-QP, CERN

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--! @brief Counter with overflow
--!
--! Contains two architectures (width/length) which differs by the way how overflow limit is
--! specified.
entity Counter is
	generic (
		--! width of the counter in bits (used in the architecture width)
		g_Width: natural := 32;
		--! length of counting sequence (used in the architecture length)
		g_Length: positive := 256
	);
	port (
		--! clock input
		Clk_ik: in std_logic;
		--! synchronous reset
		Reset_ir: in std_logic;
		--! enables counting
		Enable_i: in std_logic;
		--! overflow flag, '1' for 1 T_CLK when overflowed
		Overflow_o: out std_logic := '0'
	);
end entity;

--! Counter with overflow limit specified by generic g_Width (limit is 2^g_Width)
architecture width of Counter is

	--! actual value of the counter
	signal Value_b: unsigned(g_Width-1 downto 0) := (others => '0');
	
begin

	--! synchronous counter with synchronous reset
	pCounter: process (Clk_ik) is
		variable NewValue_b: unsigned(g_Width-1 downto 0) := (others => '0');
	begin
		if rising_edge(Clk_ik) then
			if Reset_ir = '1' then
				NewValue_b := (others => '0');
				Overflow_o <= '0';
			else
				Overflow_o <= '0';
				if Enable_i = '1' then
					NewValue_b := Value_b + 1;
					if to_integer(NewValue_b) = 0 then
						Overflow_o <= '1';
					end if;
				end if;
			end if;
			Value_b <= NewValue_b;
		end if;
	end process pCounter;

end architecture;

--! Counter with overflow limit specified by generic g_Length (limit is g_Length)
architecture length of Counter is

	--! computes least possible width of the vector to store value X
	-- http://stackoverflow.com/a/12751341/615627
	function f_log2(x: positive) return natural is
		variable i: natural;
	begin
		i := 0;
		while (2**i < x) loop
			i := i + 1;
		end loop;
		return i;
	end function;

	--! width of the vector needed for storing actual counter value
	constant c_WidthFromLength: natural := f_log2(g_Length);
	--! overflow limit
	constant c_StopValue: unsigned(c_WidthFromLength downto 0) := to_unsigned(g_Length, c_WidthFromLength+1);

	--! actual value of the counter
	signal Value_b: unsigned(c_WidthFromLength-1 downto 0) := (others => '0');
	
begin

	--! synchronous counter with synchronous reset
	pCounter: process (Clk_ik) is
		variable NewValue_b: unsigned(c_WidthFromLength-1 downto 0) := (others => '0');
	begin
		if rising_edge(Clk_ik) then
			if Reset_ir = '1' then
				NewValue_b := (others => '0');
				Overflow_o <= '0';
			else
				Overflow_o <= '0';
				if Enable_i = '1' then
					NewValue_b := Value_b + 1;
					if NewValue_b = c_StopValue(c_WidthFromLength-1 downto 0) then
						Overflow_o <= '1';
						NewValue_b := (others => '0');
					end if;
				end if;
			end if;
			Value_b <= NewValue_b;
		end if;
	end process pCounter;

end architecture;