library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity DelayController is
    generic (
        CLK_FREQ: real -- input clock frequency in Hz
    );
    port (
        clk: in std_logic;
        reset: in std_logic;
        ch1_set_value: in unsigned(9 downto 0);
        ch1_set_load: in std_logic;
        ch1_reset_value: in unsigned(9 downto 0);
        ch1_reset_load: in std_logic;
        ch2_set_value: in unsigned(9 downto 0);
        ch2_set_load: in std_logic;
        ch2_reset_value: in unsigned(9 downto 0);
        ch2_reset_load: in std_logic;
        d: out unsigned(9 downto 0);
        ch1_le_set: out std_logic;
        ch1_le_res: out std_logic;
        ch2_le_set: out std_logic;
        ch2_le_res: out std_logic;
        busy: out std_logic
    );
end entity;

architecture syn of DelayController is

    type t_output is (NONE, CH1_SET, CH1_RESET, CH2_SET, CH2_RESET);

    signal startConfiguration: std_logic := '0';
    signal loadPulse: std_logic := '0';
    signal activeOutput: t_output := NONE;

begin

    pStart: process (clk) is begin
        if rising_edge(clk) then
            if reset then
                activeOutput <= NONE;
                d <= (others => '0');
                startConfiguration <= '0';
            elsif not loadPulse then
                startConfiguration <= '0';
                if ch1_set_load then
                    activeOutput <= CH1_SET;
                    d <= ch1_set_value;
                    startConfiguration <= '1';
                end if;
                if ch1_reset_load then
                    activeOutput <= CH1_RESET;
                    d <= ch1_reset_value;
                    startConfiguration <= '1';
                end if;
                if ch2_set_load then
                    activeOutput <= CH2_SET;
                    d <= ch2_set_value;
                    startConfiguration <= '1';
                end if;
                if ch2_reset_load then
                    activeOutput <= CH2_RESET;
                    d <= ch2_reset_value;
                    startConfiguration <= '1';
                end if;
            end if;
        end if;
    end process;

    cPulseGenerator: entity work.pulseGeneratorTime(syn)
        generic map (
            CLK_FREQ => CLK_FREQ,
            PULSE_MIN_WIDTH_TIME => 1 ns
        )
        port map (
            clk => clk,
            reset => reset,
            signalIn => startConfiguration,
            pulseOut => loadPulse
        );
    
    ch1_le_set <=
        '0' when (activeOutput = CH1_SET) and loadPulse = '1' else
        '1';
    ch1_le_res <=
        '0' when (activeOutput = CH1_RESET) and loadPulse = '1' else
        '1';
    ch2_le_set <=
        '0' when (activeOutput = CH2_SET) and loadPulse = '1' else
        '1';
    ch2_le_res <=
        '0' when (activeOutput = CH2_RESET) and loadPulse = '1' else
        '1';

    busy <= loadPulse;

end architecture;
