-- TODO:
--   - trigger latency
--   - modes
--   - synchronous to SerialClock_ik


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity DelayedPulseGenerator is
    port (
        -- main signals
        Clk_ik: in std_logic;
        Reset_ir: in std_logic;
        -- trigger
        Trigger_i: in std_logic;
        -- configuration (WB)
        SetMemAddress_ob11: out unsigned(10 downto 0);
        SetMemData_ib32: in unsigned(31 downto 0);
        SetMemReadStrobe_o: out std_logic;
        ResMemAddress_ob11: out unsigned(10 downto 0);
        ResMemData_ib32: in unsigned(31 downto 0);
        ResMemReadStrobe_o: out std_logic;
        Overflow_ib16: in unsigned(15 downto 0);
        TriggerLatency_ib16: in unsigned(15 downto 0);
        Mode_ib2: in std_logic_vector(1 downto 0);
        Running_o: out std_logic;
        -- generation (FMC), synchronous to SerialClock_ik
        SerialClock_ik: in std_logic;
        Set_o: out std_logic;
        Reset_o: out std_logic
    );
end entity;

architecture syn of DelayedPulseGenerator is

    signal Running: std_logic;
    signal GeneratorReset: std_logic;
    signal MemAddress: unsigned(SetMemAddress_ob11'range);
    signal LoadShiftRegister: std_logic;
    signal StreamPosition: unsigned(Overflow_ib16'range);
    signal StreamReset: std_logic;

begin

    Running <=
        '1' when Mode_ib2 = "01" else
        '0';

    GeneratorReset <= Reset_ir or Trigger_i or StreamReset;
    
    cBitCounter: entity work.Counter(length)
        generic map (
            g_Length => SetMemData_ib32'length
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => GeneratorReset,
            Enable_i => Running,
            Overflow_o => LoadShiftRegister
        );
    
    cStreamCounter: entity work.CounterO(syn)
        generic map (
            g_Width => Overflow_ib16'length
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => Trigger_i,
            Enable_i => Running,
            Overflow_o => open,
            Value_ob => StreamPosition
        );
    
    StreamReset <=
        '1' when StreamPosition = Overflow_ib16 else
        '0';

    cAddressCounter: entity work.CounterO(syn)
        generic map (
            g_Width => SetMemAddress_ob11'length
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => GeneratorReset,
            Enable_i => LoadShiftRegister,
            Overflow_o => open,
            Value_ob => MemAddress
        );
    
    SetMemAddress_ob11 <= MemAddress;
    ResMemAddress_ob11 <= MemAddress;
    
    SetMemReadStrobe_o <= LoadShiftRegister;
    ResMemReadStrobe_o <= LoadShiftRegister;
    
    cShiftRegisterSet: entity work.ShiftRegister(right)
        generic map (
            g_Width => SetMemData_ib32'length
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => GeneratorReset,
            Data_ib => SetMemData_ib32,
            Data_ob => open,
            Shift_i => '0',
            Shift_o => Set_o,
            ShiftEnable_i => Running,
            LoadEnable_i => LoadShiftRegister
        );
    
    cShiftRegisterReset: entity work.ShiftRegister(right)
        generic map (
            g_Width => ResMemData_ib32'length
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => GeneratorReset,
            Data_ib => ResMemData_ib32,
            Data_ob => open,
            Shift_i => '0',
            Shift_o => Reset_o,
            ShiftEnable_i => Running,
            LoadEnable_i => LoadShiftRegister
        );
    
    Running_o <= Running;

end architecture;