-- TODO:
--   - automatic clock infrastructure configuration
--   - clock selection
--   - clock divider
--   - temperature somehow
--   - frequency sense
--   - LED outputs

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.FfpgPkg.all;
use work.ffpg_wbgen2_pkg.all;

entity FfpgCore is
    generic (
        g_ClkFrequency: natural -- input clock frequency in MHz
    );
    port (
        -- Wishbone connection
        Wb_i: in t_WbInterfaceMosi;
        Wb_o: out t_WbInterfaceMiso;
        --- FMC interface
        -- DACs
        TriggerDac_o: out t_Ad5600Interface;
        VcxoDac_o: out t_Ad5600Interface;
        -- output enable
        ch1_out_en: out std_logic;
        ch2_out_en: out std_logic;
        -- delay configuration
        d: out unsigned(9 downto 0);
        ch_1_le_set: out std_logic;
        ch_1_le_res: out std_logic;
        ch_2_le_set: out std_logic;
        ch_2_le_res: out std_logic;
        Ch1Set_o: out std_logic;
        Ch1Res_o: out std_logic;
        Ch2Set_o: out std_logic;
        Ch2Res_o: out std_logic;
        Trigger_i: in std_logic
    );
end entity;

architecture syn of FfpgCore is

    signal Clk, Reset: std_logic;
    
    signal SerialStreamClock: std_logic;
    
    signal Ch1SetMemAddress_b11: unsigned(10 downto 0);
    signal Ch1SetMemData_b32: unsigned(31 downto 0);
    signal Ch1SetMemReadStrobe: std_logic;
    signal Ch1ResMemAddress_b11: unsigned(10 downto 0);
    signal Ch1ResMemData_b32: unsigned(31 downto 0);
    signal Ch1ResMemReadStrobe: std_logic;
    signal Ch2SetMemAddress_b11: unsigned(10 downto 0);
    signal Ch2SetMemData_b32: unsigned(31 downto 0);
    signal Ch2SetMemReadStrobe: std_logic;
    signal Ch2ResMemAddress_b11: unsigned(10 downto 0);
    signal Ch2ResMemData_b32: unsigned(31 downto 0);
    signal Ch2ResMemReadStrobe: std_logic;

    signal WbRegsInput: t_ffpg_in_registers;
    signal WbRegsOutput: t_ffpg_out_registers;
    
begin

    Clk <= Wb_i.Clk;
    Reset <= Wb_i.Rst;
    
    -- TODO - simulation workaround until real dual-clock will be implemented
    SerialStreamClock <= Clk;

    ----------------------------------
    -- Wishbone slave
    ----------------------------------
    cWbSlaveWrapper: entity work.WbSlaveWrapper(syn)
        port map (
            Wb_i => Wb_i,
            Wb_o => Wb_o,
            SerialStreamClock_i => SerialStreamClock,
            Ch1SetMemAddress_ib11 => Ch1SetMemAddress_b11,
            Ch1SetMemData_ob32 => Ch1SetMemData_b32,
            Ch1SetMemReadStrobe_i => Ch1SetMemReadStrobe,
            Ch1ResMemAddress_ib11 => Ch1ResMemAddress_b11,
            Ch1ResMemData_ob32 => Ch1ResMemData_b32,
            Ch1ResMemReadStrobe_i => Ch1ResMemReadStrobe,
            Ch2SetMemAddress_ib11 => Ch2SetMemAddress_b11,
            Ch2SetMemData_ob32 => Ch2SetMemData_b32,
            Ch2SetMemReadStrobe_i => Ch2SetMemReadStrobe,
            Ch2ResMemAddress_ib11 => Ch2ResMemAddress_b11,
            Ch2ResMemData_ob32 => Ch2ResMemData_b32,
            Ch2ResMemReadStrobe_i => Ch2ResMemReadStrobe,
            WbRegs_i => WbRegsInput,
            WbRegs_o => WbRegsOutput
        );
    
    ----------------------------------
    -- DACs
    ----------------------------------
    cDacsController: entity work.DacsController(syn)
        generic map (
            g_ClkFrequency => g_ClkFrequency
        )
        port map (
            Clk_ik => Clk,
            Reset_ir => Reset,
            TriggerValue_ib16 => WbRegsOutput.trigger_threshold_o,
            TriggerLoad_i => WbRegsOutput.trigger_threshold_load_o,
            VcxoValue_ib16 => WbRegsOutput.vcxo_voltage_o,
            VcxoLoad_i => WbRegsOutput.vcxo_voltage_load_o,
            TriggerDac_o => TriggerDac_o,
            VcxoDac_o => VcxoDac_o,
            TriggerBusy_o => WbRegsInput.status_dac_trigger_busy_i,
            VcxoBusy_o => WbRegsInput.status_dac_vcxo_busy_i
        );

    ----------------------------------
    -- channel delay configuration
    ----------------------------------
    cDelayController: entity work.DelayController(syn)
        generic map (
            CLK_FREQ => real(g_ClkFrequency)*1.0e6
        )
        port map (
            clk => Clk,
            reset => Reset,
            ch1_set_value => WbRegsOutput.ch1_delay_set_o,
            ch1_set_load => WbRegsOutput.ch1_delay_set_load_o,
            ch1_reset_value => WbRegsOutput.ch1_delay_reset_o,
            ch1_reset_load => WbRegsOutput.ch1_delay_reset_load_o,
            ch2_set_value => WbRegsOutput.ch2_delay_set_o,
            ch2_set_load => WbRegsOutput.ch2_delay_set_load_o,
            ch2_reset_value => WbRegsOutput.ch2_delay_reset_o,
            ch2_reset_load => WbRegsOutput.ch2_delay_reset_load_o,
            d => d,
            ch1_le_set => ch_1_le_set,
            ch1_le_res => ch_1_le_res,
            ch2_le_set => ch_2_le_set,
            ch2_le_res => ch_2_le_res,
            busy => WbRegsInput.status_delay_configuration_busy_i
        );

    ----------------------------------
    -- output enable controls
    ----------------------------------
    ch1_out_en <= WbRegsOutput.control_ch1_oe_o;
    ch2_out_en <= WbRegsOutput.control_ch2_oe_o;
    WbRegsInput.status_channel_1_oe_i <= WbRegsOutput.control_ch1_oe_o;
    WbRegsInput.status_channel_2_oe_i <= WbRegsOutput.control_ch2_oe_o;

    ----------------------------------
    -- serial stream generators
    ----------------------------------
    cDelayedPulseGeneratorCh1: entity work.DelayedPulseGenerator(syn)
        port map (
            Clk_ik => Clk,
            Reset_ir => Reset,
            Trigger_i => Trigger_i,
            SetMemAddress_ob11 => Ch1SetMemAddress_b11,
            SetMemData_ib32 => Ch1SetMemData_b32,
            SetMemReadStrobe_o => Ch1SetMemReadStrobe,
            ResMemAddress_ob11 => Ch1ResMemAddress_b11,
            ResMemData_ib32 => Ch1ResMemData_b32,
            ResMemReadStrobe_o => Ch1ResMemReadStrobe,
            Overflow_ib16 => WbRegsOutput.overflow_o,
            TriggerLatency_ib16 => WbRegsOutput.trigger_latency_o,
            Mode_ib2 => WbRegsOutput.control_ch1_mode_o,
            Running_o => WbRegsInput.status_channel_1_running_i,
            SerialClock_ik => SerialStreamClock,
            Set_o => Ch1Set_o,
            Reset_o => Ch1Res_o
        );

    cDelayedPulseGeneratorCh2: entity work.DelayedPulseGenerator(syn)
        port map (
            Clk_ik => Clk,
            Reset_ir => Reset,
            Trigger_i => Trigger_i,
            SetMemAddress_ob11 => Ch2SetMemAddress_b11,
            SetMemData_ib32 => Ch2SetMemData_b32,
            SetMemReadStrobe_o => Ch2SetMemReadStrobe,
            ResMemAddress_ob11 => Ch2ResMemAddress_b11,
            ResMemData_ib32 => Ch2ResMemData_b32,
            ResMemReadStrobe_o => Ch2ResMemReadStrobe,
            Overflow_ib16 => WbRegsOutput.overflow_o,
            TriggerLatency_ib16 => WbRegsOutput.trigger_latency_o,
            Mode_ib2 => WbRegsOutput.control_ch2_mode_o,
            Running_o => WbRegsInput.status_channel_2_running_i,
            SerialClock_ik => SerialStreamClock,
            Set_o => Ch2Set_o,
            Reset_o => Ch2Res_o
        );

    ----------------------------------
    -- to be used
    ----------------------------------
    -- WbRegsOutput.clock_divider_lo_o
    WbRegsInput.status_clock_infrastructure_busy_i <= 'X';
    WbRegsInput.status_clock_selection_busy_i <= 'X';
    WbRegsInput.frequency_i <= (others => 'X');

end architecture;
