library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.FfpgPkg.all;
use work.ffpg_wbgen2_pkg.all;

entity WbSlaveWrapper is
    port (
        Wb_i: in t_WbInterfaceMosi;
        Wb_o: out t_WbInterfaceMiso;
        SerialStreamClock_i: in std_logic;
        Ch1SetMemAddress_ib11: in unsigned(10 downto 0);
        Ch1SetMemData_ob32: out unsigned(31 downto 0);
        Ch1SetMemReadStrobe_i: in std_logic;
        Ch1ResMemAddress_ib11: in unsigned(10 downto 0);
        Ch1ResMemData_ob32: out unsigned(31 downto 0);
        Ch1ResMemReadStrobe_i: in std_logic;
        Ch2SetMemAddress_ib11: in unsigned(10 downto 0);
        Ch2SetMemData_ob32: out unsigned(31 downto 0);
        Ch2SetMemReadStrobe_i: in std_logic;
        Ch2ResMemAddress_ib11: in unsigned(10 downto 0);
        Ch2ResMemData_ob32: out unsigned(31 downto 0);
        Ch2ResMemReadStrobe_i: in std_logic;
        WbRegs_i: in t_ffpg_in_registers;
        WbRegs_o: out t_ffpg_out_registers
    );
end entity;

architecture syn of WbSlaveWrapper is

    signal WbRegsInput: t_ffpg_in_registers;
    signal WbRegsOutput: t_ffpg_out_registers;

    -- registers for LOAD_EXT fields
    signal control_clock_selection: std_logic_vector(1 downto 0);
    signal control_ch1_mode: std_logic_vector(1 downto 0);
    signal control_ch2_mode: std_logic_vector(1 downto 0);
    signal vcxo_voltage: unsigned(15 downto 0);
    signal clock_divider_hi: unsigned(3 downto 0);
    signal ch1_delay_set: unsigned(9 downto 0);
    signal ch1_delay_reset: unsigned(9 downto 0);
    signal ch2_delay_set: unsigned(9 downto 0);
    signal ch2_delay_reset: unsigned(9 downto 0);
    signal trigger_threshold: unsigned(15 downto 0);
    signal overflow: unsigned(15 downto 0);
    -- delayed load signals for LOAD_EXT fields
    signal control_clock_selection_load: std_logic;
    signal control_ch1_mode_load: std_logic;
    signal control_ch2_mode_load: std_logic;
    signal vcxo_voltage_load: std_logic;
    signal clock_divider_hi_load: std_logic;
    signal ch1_delay_set_load: std_logic;
    signal ch1_delay_reset_load: std_logic;
    signal ch2_delay_set_load: std_logic;
    signal ch2_delay_reset_load: std_logic;
    signal trigger_threshold_load: std_logic;
    signal overflow_load: std_logic;
    
begin

    cWbSlave: entity work.ffpg_csr(syn)
        port map (
            rst_n_i => not Wb_i.Rst,
            clk_sys_i => Wb_i.Clk,
            wb_adr_i => Wb_i.Adr_b32(13 downto 0),
            wb_dat_i => Wb_i.Dat_b32,
            wb_dat_o => Wb_o.Dat_b32,
            wb_cyc_i => Wb_i.Cyc,
            wb_sel_i => Wb_i.Sel_b4,
            wb_stb_i => Wb_i.Stb,
            wb_we_i => Wb_i.We,
            wb_ack_o => Wb_o.Ack,
            wb_stall_o => Wb_o.Stall,
            serial_stream_clk_ik => SerialStreamClock_i,
            ffpg_ch1_set_mem_addr_i => std_logic_vector(Ch1SetMemAddress_ib11),
            unsigned(ffpg_ch1_set_mem_data_o) => Ch1SetMemData_ob32,
            ffpg_ch1_set_mem_rd_i => Ch1SetMemReadStrobe_i,
            ffpg_ch1_res_mem_addr_i => std_logic_vector(Ch1ResMemAddress_ib11),
            unsigned(ffpg_ch1_res_mem_data_o) => Ch1ResMemData_ob32,
            ffpg_ch1_res_mem_rd_i => Ch1ResMemReadStrobe_i,
            ffpg_ch2_set_mem_addr_i => std_logic_vector(Ch2SetMemAddress_ib11),
            unsigned(ffpg_ch2_set_mem_data_o) => Ch2SetMemData_ob32,
            ffpg_ch2_set_mem_rd_i => Ch2SetMemReadStrobe_i,
            ffpg_ch2_res_mem_addr_i => std_logic_vector(Ch2ResMemAddress_ib11),
            unsigned(ffpg_ch2_res_mem_data_o) => Ch2ResMemData_ob32,
            ffpg_ch2_res_mem_rd_i => Ch2ResMemReadStrobe_i,
            regs_i => WbRegsInput,
            regs_o => WbRegsOutput
        );

    -- local registers for LOAD_EXT fields
    pLocalRegs: process (Wb_i.Clk) is begin
        if rising_edge(Wb_i.Clk) then
            control_clock_selection_load <= '0';
            control_ch1_mode_load <= '0';
            control_ch2_mode_load <= '0';
            vcxo_voltage_load <= '0';
            clock_divider_hi_load <= '0';
            ch1_delay_set_load <= '0';
            ch1_delay_reset_load <= '0';
            ch2_delay_set_load <= '0';
            ch2_delay_reset_load <= '0';
            trigger_threshold_load <= '0';
            overflow_load <= '0';

            if WbRegsOutput.control_clock_selection_load_o then
                control_clock_selection <= WbRegsOutput.control_clock_selection_o;
                control_clock_selection_load <= '1';
            end if;
            if WbRegsOutput.control_ch1_mode_load_o then
                control_ch1_mode <= WbRegsOutput.control_ch1_mode_o;
                control_ch1_mode_load <= '1';
            end if;
            if WbRegsOutput.control_ch2_mode_load_o then
                control_ch2_mode <= WbRegsOutput.control_ch2_mode_o;
                control_ch2_mode_load <= '1';
            end if;
            if WbRegsOutput.vcxo_voltage_load_o then
                vcxo_voltage <= WbRegsOutput.vcxo_voltage_o;
                vcxo_voltage_load <= '1';
            end if;
            if WbRegsOutput.clock_divider_hi_load_o then
                clock_divider_hi <= WbRegsOutput.clock_divider_hi_o;
                clock_divider_hi_load <= '1';
            end if;
            if WbRegsOutput.ch1_delay_set_load_o then
                ch1_delay_set <= WbRegsOutput.ch1_delay_set_o;
                ch1_delay_set_load <= '1';
            end if;
            if WbRegsOutput.ch1_delay_reset_load_o then
                ch1_delay_reset <= WbRegsOutput.ch1_delay_reset_o;
                ch1_delay_reset_load <= '1';
            end if;
            if WbRegsOutput.ch2_delay_set_load_o then
                ch2_delay_set <= WbRegsOutput.ch2_delay_set_o;
                ch2_delay_set_load <= '1';
            end if;
            if WbRegsOutput.ch2_delay_reset_load_o then
                ch2_delay_reset <= WbRegsOutput.ch2_delay_reset_o;
                ch2_delay_reset_load <= '1';
            end if;
            if WbRegsOutput.trigger_threshold_load_o then
                trigger_threshold <= WbRegsOutput.trigger_threshold_o;
                trigger_threshold_load <= '1';
            end if;
            if WbRegsOutput.overflow_load_o then
                overflow <= WbRegsOutput.overflow_o;
                overflow_load <= '1';
            end if;
        end if;
    end process;
    
    pInputRegisters: process (all) is begin
        -- by default, all values are passed
        WbRegsInput <= WbRegs_i;
        -- LOAD_EXT inputs are overwritten by local registers
        WbRegsInput.control_clock_selection_i <= control_clock_selection;
        WbRegsInput.control_ch1_mode_i <= control_ch1_mode;
        WbRegsInput.control_ch2_mode_i <= control_ch2_mode;
        WbRegsInput.vcxo_voltage_i <= vcxo_voltage;
        WbRegsInput.clock_divider_hi_i <= clock_divider_hi;
        WbRegsInput.ch1_delay_set_i <= ch1_delay_set;
        WbRegsInput.ch1_delay_reset_i <= ch1_delay_reset;
        WbRegsInput.ch2_delay_set_i <= ch2_delay_set;
        WbRegsInput.ch2_delay_reset_i <= ch2_delay_reset;
        WbRegsInput.trigger_threshold_i <= trigger_threshold;
        WbRegsInput.overflow_i <= overflow;
    end process;
    
    pOutputRegisters: process (all) is begin
        -- by default, all values are passed
        WbRegs_o <= WbRegsOutput;
        -- LOAD_EXT inputs are overwritten by local registers
        WbRegs_o.control_clock_selection_o <= control_clock_selection;
        WbRegs_o.control_ch1_mode_o <= control_ch1_mode;
        WbRegs_o.control_ch2_mode_o <= control_ch2_mode;
        WbRegs_o.vcxo_voltage_o <= vcxo_voltage;
        WbRegs_o.clock_divider_hi_o <= clock_divider_hi;
        WbRegs_o.ch1_delay_set_o <= ch1_delay_set;
        WbRegs_o.ch1_delay_reset_o <= ch1_delay_reset;
        WbRegs_o.ch2_delay_set_o <= ch2_delay_set;
        WbRegs_o.ch2_delay_reset_o <= ch2_delay_reset;
        WbRegs_o.trigger_threshold_o <= trigger_threshold;
        WbRegs_o.overflow_o <= overflow;
        WbRegs_o.control_clock_selection_load_o <= control_clock_selection_load;
        WbRegs_o.control_ch1_mode_load_o <= control_ch1_mode_load;
        WbRegs_o.control_ch2_mode_load_o <= control_ch2_mode_load;
        WbRegs_o.vcxo_voltage_load_o <= vcxo_voltage_load;
        WbRegs_o.clock_divider_hi_load_o <= clock_divider_hi_load;
        WbRegs_o.ch1_delay_set_load_o <= ch1_delay_set_load;
        WbRegs_o.ch1_delay_reset_load_o <= ch1_delay_reset_load;
        WbRegs_o.ch2_delay_set_load_o <= ch2_delay_set_load;
        WbRegs_o.ch2_delay_reset_load_o <= ch2_delay_reset_load;
        WbRegs_o.trigger_threshold_load_o <= trigger_threshold_load;
        WbRegs_o.overflow_load_o <= overflow_load;
    end process;

end architecture;